@ECHO OFF
rem USO: BackupBaseDatos

rem ----------------------------------------
rem -- parametros del proceso 
rem -- DB_NAME: nombre BaseDatos; BK_CARPETA: carpeta archivos de salida
rem ----------------------------------------
SET DB_NAME=nombrebasedatos
SET BK_CARPETA=C:\Abaco\Backups\Data

rem -- Resguardar carpeta indicada para informar en .log
SET PBK_CARPETA=%BK_CARPETA%
SET BK_EXISTECARPETA=0
rem ---------------------------------------------------------------------------------------------------------
rem -- Verificar que exista la carpeta para los archivos de salida.
rem -- Si no existe, se recupera la carpeta donde se esta ejecutando el .bat 
rem -- Si no existe, BK_EXISTECARPETA=6 (para setear abmcfg.corrupta) y
rem    se establece la salida a <carpeta del .bat>\Data
rem -- Para extraer la ruta del archivo .bat -> %~dp0
rem ---------------------------------------------------------------------------------------------------------
IF NOT EXIST %BK_CARPETA% (
  SET BK_EXISTECARPETA=6
  SET BK_CARPETA=%~dp0Data
)

rem ------------------------------------------------------------------------------------
rem -- Carpeta para salidas del proceso. Si no existe se crea  --
rem ------------------------------------------------------------------------------------
IF NOT EXIST "%BK_CARPETA%" MKDIR "%BK_CARPETA%"

rem ------------------------------------------
rem -- Parametros de la conexion de la base --
rem ------------------------------------------
rem ** nombre o IP del host  de la base de datos **
rem SET DB_HOST=LocalHost
rem SET DB_HOST=127.0.0.1
SET DB_HOST=LOCALHOST

rem ** nombre del puerto del servidor **
rem SET DB_PORT=5432
SET DB_PORT=5432

rem ---------------------------------------------------------------------------------------
rem -- Carpeta con comandos postgres para backup y update   --
rem ---------------------------------------------------------------------------------------
rem SET PGSQL_PATH=C:\Abaco\pgsql9\bin
SET PGSQL_PATH=C:\Abaco\pgsql9\bin

rem ----------------------------------------
rem **Archivos de salida **
rem ----------------------------------------
rem -- Agrego la fecha --
FOR /F "tokens=1,2,3 delims=/ " %%A IN ('date /t') DO (SET TIMESTAMP=%%C-%%B-%%A)
FOR /F "tokens=1,2   delims=:" %%A IN ('time /t')  DO (SET TIMESTAMP=%TIMESTAMP% %%A-%%B)

rem ** nombre de los archivos de salida **
SET WK_ARCHIVO=%DB_NAME%_%TIMESTAMP%

rem ** Archivo de backup **
SET BK_FILE=%BK_CARPETA%\%WK_ARCHIVO%.sql

rem ** Archivo de mensajes de salida **
SET BK_LOG=%BK_CARPETA%\%WK_ARCHIVO%.log

rem ** Archivo de backup con errores (no incluye path porque se usa en comanado RENAME) **
SET BK_MAL=%WK_ARCHIVO%.mal

rem ----------------------------------------------
rem -- Opciones para el pg_dump  --
rem ----------------------------------------------
rem ** Excluir tablas del backup **
rem ** Sintaxis general: -T tabla1 -T tabla2 .. -T tablaN Se pueden usar expresiones regurales ej. -T pad* para no backup de tablas que comiencen con pad  **
SET OP_EXCLUIRTABLAS=
rem SET OP_EXCLUIRTABLAS= -T padronrentas

rem ** Formato texto plano **
rem SET OP_FORMATO= -F plain
SET OP_FORMATO=

rem ** Formato encodig **
rem SET OP_ENCODING= -E SQL_ASCII
SET OP_ENCODING=

rem -----------------------------------------------------------------------------------------------
rem -- Valores para actualizar ambcfg.corrupta al finalizar el backup  --
rem -- backup con error: ambcfg.corrupta = 1
rem -- backup correcto:  ambcfg.corrupta = 0
rem -- backup correcto, carpeta salida de archivos no existe  -> ambcfg.corrupta = 6
rem -----------------------------------------------------------------------------------------------
SET PGCMD_BKERROR="SELECT soporte.actualizarestadobackup(1)"
SET PGCMD_BKEXITOSO="SELECT soporte.actualizarestadobackup(%BK_EXISTECARPETA%)"

ECHO.
ECHO *************************************************************
ECHO    Inicio backup de la base de datos "%DB_NAME%"
ECHO *************************************************************
ECHO.
ECHO realizando backup ...

rem ** Comando postgres que realiza el backup **
"%PGSQL_PATH%"\pg_dump.exe -O -U abacouser -p "%DB_PORT%" -h "%DB_HOST%" %OP_FORMATO% %OP_ENCODING% %OP_EXCLUIRTABLAS%  %DB_NAME%> "%BK_FILE%" 2>"%BK_LOG%"

IF %ERRORLEVEL% NEQ 0 (
  rem -- Si hubo un error durante el backup, recuperar informacion en .log
  ECHO. >> "%BK_LOG%"
  ECHO Cdigo retorno pg_dump.exe:  %ERRORLEVEL% >> "%BK_LOG%"
  ECHO Carpeta comandos postgresql: "%PGSQL_PATH%" >> "%BK_LOG%"
  ECHO Carpeta indicada para los archivos de salida: "%PBK_CARPETA%" >> "%BK_LOG%"
  ECHO Conexin base datos:  HOST="%DB_HOST%" PORT="%DB_PORT%" >> "%BK_LOG%"
  ECHO Base datos: %DB_NAME% >> "%BK_LOG%"
  IF %BK_EXISTECARPETA% NEQ 0 (
    ECHO La carpeta "%PBK_CARPETA%" no existe. Los archivos se generaron en "%BK_CARPETA%">> "%BK_LOG%" 
  )
  ECHO. >> "%BK_LOG%"

  rem ** Actualizacion del estado de backup INCORRECTO (ambcfg.corrupta=1)***
  "%PGSQL_PATH%\psql.exe" -U abacouser -p "%DB_PORT%" -h "%DB_HOST%" -d %DB_NAME%  -c %PGCMD_BKERROR%  1>>"%BK_LOG%" 2>>&1

  rem Renombrar backup malo
  IF EXIST "%BK_MAL%" DEL "%BK_MAL%"
  RENAME "%BK_FILE%" "%BK_MAL%"

  ECHO.
  ECHO ************************************************
  ECHO ******** NO SE PUDO CONCRETAR EL BACKUP ********
  ECHO ************************************************
  ECHO.
  GOTO :FIN
)

rem Este es el ELSE del IF (%ERRORLEVEL = 0%) 
rem -- actualizacion del estado de backup correcto <> ambcfg.corrupta=0; fechaultimobackup=now() --
"%PGSQL_PATH%\psql.exe" -U abacouser -p "%DB_PORT%" -h "%DB_HOST%" -d %DB_NAME% -c %PGCMD_BKEXITOSO%  1>>"%BK_LOG%" 2>>&1

IF %ERRORLEVEL% NEQ 0 (
  rem -- datos de la conexion
  ECHO Cdigo retorno psql.exe:  %ERRORLEVEL% >> "%BK_LOG%"
  ECHO Conexin base datos:  HOST="%DB_HOST%" PORT="%DB_PORT%" >> "%BK_LOG%"
  ECHO Base datos: %DB_NAME% >> "%BK_LOG%"
  ECHO. >> "%BK_LOG%"
  ECHO.
  ECHO ************************************************  
  ECHO ******** NO SE PUDO ACTUALIZAR ESTADO BASE DATOS ********
  ECHO ************************************************
  ECHO.
  GOTO :FIN
)
rem --Informar si la carpeta indicada como salida era correcta
IF %BK_EXISTECARPETA% NEQ 0 (
  ECHO La carpeta "%PBK_CARPETA%" indicada para la salida de archivos no existe>> "%BK_LOG%" 
  ECHO Los archivos se generaron en "%BK_CARPETA%" >> "%BK_LOG%" 
  GOTO :BK_FINAL  
)

rem --si no hubo error al actualizar la BD -> borrar el .log
DEL "%BK_LOG%"

:BK_FINAL
ECHO.
ECHO ***************************
ECHO **** BACKUP FINALIZADO ****
ECHO ***************************
GOTO :FIN

rem  -- FIN DEL PROCESO --
:FIN
rem --borrar variables de entorno creadas por el proceso
SET DB_NAME=
SET BK_CARPETA=
SET PBK_CARPETA=
SET BK_EXISTECARPETA=
SET DB_HOST=
SET DB_PORT=
SET PGSQL_PATH=
SET WK_ARCHIVO=
SET BK_FILE=
SET BK_LOG=
SET BK_MAL=
ECHO.
pause
